"use strict";

exports.__esModule = true;
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _isPromise = _interopRequireDefault(require("is-promise"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var DefaultLoadingComponent = function DefaultLoadingComponent() {
  return null;
};

var DefaultErrorComponent = function DefaultErrorComponent() {
  return null;
};

var AsyncComponent =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(AsyncComponent, _Component);

  function AsyncComponent() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _Component.call.apply(_Component, [this].concat(args)) || this;
    _this.errorComponent = null;
    _this.state = {
      error: null,
      loading: true,
      resolvedProps: {}
    };
    return _this;
  }

  var _proto = AsyncComponent.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.mounted = true;
    this.load();
  };

  _proto.componentWillUpdate = function componentWillUpdate() {
    this.mounted = false;
  };

  _proto.load = function load() {
    var _this2 = this;

    var pairs = (0, _utils.toPairs)(this.props.asyncProps || {}).filter(function (entry) {
      return entry[1];
    });

    if ((0, _utils.some)(pairs, function (entry) {
      return !(0, _utils.isFunction)(entry[1]);
    })) {
      this.setState({
        error: new Error('The prop of asyncProps must be a function.'),
        loading: false
      });
      return;
    }

    var batch = this.props.batch;
    var promises = pairs.map(function (_ref) {
      var k = _ref[0],
          v = _ref[1];
      return new Promise(function (resolve) {
        return resolve([k, v()]);
      }).then(function (_ref2) {
        var k = _ref2[0],
            p = _ref2[1];

        if ((0, _isPromise.default)(p)) {
          return p.then(function (res) {
            if (!batch) {
              if (_this2.mounted) {
                _this2.setState(AsyncComponent.updateResolvedProps({
                  k: res
                }));
              } else {
                _this2.state.resolvedProps[k] = res;
              }
            }

            return [k, res];
          });
        }

        return [k, p];
      });
    });
    Promise.all(promises).then(function (pairs) {
      if (_this2.mounted) {
        _this2.setState({
          resolvedProps: (0, _utils.fromPairs)(pairs),
          error: null
        });
      } else {
        _this2.state.resolvedProps = (0, _utils.fromPairs)(pairs);
        _this2.state.error = null;
      }
    }).catch(function (e) {
      if (_this2.mounted) {
        _this2.setState({
          error: e
        });
      } else {
        _this2.state.error = e;
      }
    }).finally(function () {
      if (_this2.mounted) {
        _this2.setState({
          loading: false
        });
      } else {
        _this2.state.loading = false;
      }
    });
  };

  _proto.render = function render() {
    var _props = this.props,
        ErrorComponent = _props.errorComponent,
        LoadingComponent = _props.loadingComponent,
        Comp = _props.component,
        children = _props.children;
    var _state = this.state,
        error = _state.error,
        loading = _state.loading,
        resolvedProps = _state.resolvedProps;

    if (loading) {
      return _react.default.createElement(LoadingComponent, _extends({}, this.props, resolvedProps));
    }

    if (error) {
      return _react.default.createElement(ErrorComponent, _extends({}, this.props, resolvedProps, {
        error: error
      }));
    }

    if (Comp) {
      return _react.default.createElement(Comp, _extends({}, this.props, resolvedProps));
    } // noinspection JSUnresolvedFunction JSCheckFunctionSignatures


    var newChildren = _react.default.Children.map(children, function (child) {
      return _react.default.cloneElement(child, resolvedProps);
    });

    if (newChildren.length === 1) {
      return newChildren[0];
    } else if (newChildren.length > 1) {
      return _react.default.createElement("div", null, newChildren);
    }

    return null;
  };

  return AsyncComponent;
}(_react.Component);

AsyncComponent.updateResolvedProps = function (resolvedProps) {
  return function (preState) {
    return {
      resolvedProps: _objectSpread({}, preState.resolvedProps, resolvedProps)
    };
  };
};

var bool = _propTypes.default.bool,
    object = _propTypes.default.object,
    func = _propTypes.default.func,
    oneOfType = _propTypes.default.oneOfType,
    arrayOf = _propTypes.default.arrayOf,
    element = _propTypes.default.element;
AsyncComponent.propTypes = {
  batch: bool,
  // eslint-disable-next-line react/forbid-prop-types
  asyncProps: object.isRequired,
  component: func,
  children: oneOfType([element, arrayOf(element)]),
  errorComponent: func,
  loadingComponent: func
};
AsyncComponent.defaultProps = {
  batch: false,
  component: null,
  children: null,
  errorComponent: DefaultErrorComponent,
  loadingComponent: DefaultLoadingComponent
};
var _default = AsyncComponent;
exports.default = _default;
module.exports = exports["default"];